@@ -2,7 +2,8 @@
 2015. 12. 10
 Hans Roh
 """
-__version__ = "0.2.3.1"
+
+__version__ = "0.3.2.4"
 
 version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))
 
@@ -23,9 +24,6 @@ request_tearing_down = "request:teardown"
 request_starting = "request:started"
 request_finished = "request:finished"
 
-template_rendering = "template:rendering"
-template_rendered = "template:rendered"
-
 
 def load (target, pref = None):
     from rs4 import importer
