@@ -232,6 +232,8 @@ class Auth:
                     perms = set ()
                     for k, v in specified.items ():
                         specific = was.request.ARGS.get (k)
+                        if specific == "notme" and was.request.method not in ("GET", "OPTIONS"):
+                            raise was.Error ("421 Method Not Allowed")
                         if specific and specific != "me":
                             perms = v.copy ()
                             break
