@@ -1,12 +1,13 @@
 from functools import wraps
-import os
+import os, sys
 from importlib import reload
 from skitai import was as the_was
 import time, threading
 from ..storage import Storage
 import atila
 from . import decorators, auth, template_engine, router, events
-    
+from types import ModuleType
+
 class MixIn (
         decorators.Decorators,
         router.Router, 
@@ -16,15 +17,14 @@ class MixIn (
     ):    
     use_reloader = False
     debug = False
-    auto_mount = True
-    contrib_devel = False # make reloadable
+    auto_mount = True    
     
     securekey = None
     session_timeout = None
 
     access_control_allow_origin = None
     access_control_max_age = 0
-    authenticate = None
+    authenticate = None    
     
     def __init__ (self):
         decorators.Decorators.__init__ (self)
@@ -44,18 +44,19 @@ class MixIn (
         self.path_suffix_len = 0
         self.mount_params = {}
         self.reloadables = {}
+        self.reloadable_objects = {}
         self.last_reloaded = time.time ()        
         self.init_time = time.time ()
         
         self._maintern_funcs = {}  
         self._package_dirs = []           
-        self._mount_option = {}
-        self._started = False        
+        self._mount_option = {}        
+        self._started = False
         
         self._salt = None        
         
         self.store ["__last_maintern"] = 0.0
-        self.store ["__maintern_count"] = 0
+        self.store ["__maintern_count"] = 0        
     
     def maintern (self):
         if not self._maintern_funcs:
@@ -113,7 +114,10 @@ class MixIn (
     def salt (self):
         if self._salt:
             return self._salt
-        self._salt = self.securekey.encode ("utf8")
+        if not self.securekey:
+            self._salt = None
+        else:            
+            self._salt = self.securekey.encode ("utf8")
         return self._salt
     
     def set_default_session_timeout (self, timeout):
@@ -131,22 +135,23 @@ class MixIn (
         for name in names:
             self.PACKAGE_DIRS.append (name)
     
+    MOUNT_HOOKS = ["__mount__", "mount", "decorate"]
+    UMOUNT_HOOKS = ["__umount__", "umount", "dettach"]
     def _mount (self, module):
         mount_func = None
-        if hasattr (module, "mount"):
-            mount_func = module.mount
-        elif hasattr (module, "decorate"): # for old ver
-            mount_func = module.decorate
+        for hook in self.MOUNT_HOOKS:
+            if hasattr (module, hook):
+                mount_func = getattr (module, hook)
+                break
                         
         if mount_func:
             if not self.auto_mount and module not in self.mount_params:            
-                return        
+                return
             params = self.mount_params.get (module, {})
             if params.get ("debug_only") and not self.debug:
                 return
-            # for app
-            setattr (module, "__options__", params)
-            setattr (module, "_mount__", params)
+            
+            setattr (module, "__mntopt__", params)
             # for app initialzing and reloading
             self._mount_option = params
             try:
@@ -162,31 +167,76 @@ class MixIn (
             return        
         # find recursively
         self.find_mountables (module)
-        
+
+    def _reload_objects (self, origin):
+        if origin not in self.reloadable_objects:
+            return
+
+        deletables = []    
+        for objname, includers in self.reloadable_objects [origin].items ():
+            for each in includers:
+                try:
+                    attr = getattr (origin, objname)
+                except AttributeError:                    
+                    deletables.append (objname)
+                    continue          
+                setattr (each, objname, attr)      
+
+        for objname in deletables:
+            try:
+                del self.reloadable_objects [origin][objname]
+            except KeyError:
+                pass
+
+    def _set_reloadable_object (self, objname, origin, includer):
+        if origin not in self.reloadable_objects:
+            self.reloadable_objects [origin] = {}
+        if objname not in self.reloadable_objects [origin]:
+            self.reloadable_objects [origin][objname] = set ()
+        self.reloadable_objects [origin][objname].add (includer)
+
     def find_mountables (self, module):
         for attr in dir (module):
-            v = getattr (module, attr)
+            if attr.startswith ("__"):
+                continue
+            v = getattr (module, attr)            
+            maybe_object = None
+            mountable = False
+            
+            if hasattr (v, "__module__"):
+                maybe_object = attr                
+                v = sys.modules [v.__module__]
+                if v == module:
+                    continue
+
+            if type (v) is not ModuleType:
+                continue
+            
             try:
                 modpath = v.__spec__.origin
             except AttributeError:
                 continue
+  
             if not modpath:
-                continue            
-            if v in self.reloadables:
                 continue
-            if self.contrib_devel:
-                if modpath.startswith (self.CONTRIB_DIR):
-                    self._mount (v)
-                    continue
+
+            if v in self.reloadables:
+                maybe_object and self._set_reloadable_object (maybe_object, v, module)
+                continue
+
             for package_dir in self._package_dirs:
-                if modpath.startswith (package_dir):
-                    self._mount (v)
+                if modpath.startswith (package_dir):                    
+                    mountable = True                        
                     break
+        
+            if mountable:
+                self._mount (v)                
+                maybe_object and self._set_reloadable_object (maybe_object, v, module)
     
     def add_package_dir (self, path):
         if path not in self._package_dirs:
-            self._package_dirs.append (path)
-            
+            self._package_dirs.append (path)        
+
     def mount_externals (self):
         for module in self.mount_params:            
             if module in self.reloadables:
@@ -199,23 +249,31 @@ class MixIn (
                 kargs ["point"] = maybe_point
             else:
                 modules = (maybe_point,) + modules
+
         for module in modules:
-            assert hasattr (module, "mount") or hasattr (module, "decorate")            
+            mount_func = None
+            for hook in self.MOUNT_HOOKS:
+                if hasattr (module, hook):
+                    mount_func = getattr (module, hook)
+                    break
+            assert mount_func, "__mount__ hook doesn't exist"
             self.add_package_dir (os.path.dirname (module.__spec__.origin))
             self.mount_params [module] = (kargs)
     mount_with = decorate_with = mount    
     
-    def umount (self, *modules):
+    def umount (self, *modules):        
         for module in modules:
             umount_func = None
-            if hasattr (module, "umount"):
-                umount_func = module.umount
-            elif hasattr (module, "dettach"): # for old ver
-                umount_func = module.dettach 
-            umount_func and umount_func (self)
+            for hook in self.UMOUNT_HOOKS:
+                if hasattr (module, hook):
+                    umount_func = getattr (module, hook)
+                    break
+            if not umount_func:
+                return                
+            umount_func (self)
             self.log ("- %s umounted" % module.__name__, "info")
          
-    def  umount_all (self):
+    def umount_all (self):
         self.umount (*tuple (self.reloadables.keys ()))
     dettach_all = umount_all
     
@@ -239,7 +297,8 @@ class MixIn (
                 with self.lock:
                     newmodule = reload (module)
                     del self.reloadables [module]
-                    self._mount (newmodule)
+                    self._mount (newmodule)                    
+                    self._reload_objects (newmodule)
                     
         self.load_jinja_filters ()
         self.last_reloaded = time.time ()        
