@@ -8,6 +8,7 @@ import inspect
 RX_RULE = re.compile ("(/<(.+?)>)")
 
 class Router:
+    SPECIALS = ("notme", "me", "new")
     def __init__ (self):        
         self.route_map = {}
         self.route_map_fancy = {}        
@@ -27,9 +28,9 @@ class Router:
         return decorator
     
     def add_route (self, rule, func, **options):
-        if not rule or rule [0] != "/":
-            raise AssertionError ("Url rule should be starts with '/'")
-        
+        if rule and rule [0] != "/":
+            raise AssertionError ("Url rule should be starts with '/', {} {}".format (rule))
+            
         func_id = self.get_func_id (func)
         is_alter_routing = id (func) in self._function_names
         
@@ -46,10 +47,10 @@ class Router:
                         break
                 if deletable:
                     del self._function_names [deletable]
-        
+
         mount_prefix = self._mount_option.get ("point")
         if not mount_prefix:            
-            mount_prefix = self._mount_option.get ("mount")    
+            mount_prefix = self._mount_option.get ("mount")        
                         
         if mount_prefix:            
             while mount_prefix:
@@ -116,10 +117,10 @@ class Router:
             for r, n in urlargs:
                 if n.startswith ("int:"):
                     rulenames.append ((n[4:], n[:3]))
-                    rule = rule.replace (r, "/(me|[0-9]+)")
+                    rule = rule.replace (r, "/({}|[0-9]+)".format ("|".join (self.SPECIALS)))
                 elif n.startswith ("float:"):
                     rulenames.append ((n[6:], n [:5]))
-                    rule = rule.replace (r, "/(me|[.0-9]+)")
+                    rule = rule.replace (r, "/({}|[.0-9]+)".format ("|".join (self.SPECIALS)))
                 elif n.startswith ("path:"):
                     rulenames.append ((n[5:], n [:4]))
                     rule = rule.replace (r, "/(.+)")    
@@ -139,15 +140,18 @@ class Router:
             proto = (func, func.__name__, func.__code__.co_varnames [1:func.__code__.co_argcount], tuple (rulenames), func.__code__.co_argcount - 1, s_rule, options)
             
             if s >= 0 and len (rulenames) == 1 and s_rule [-1] == ">" and rulenames [0][0] in options.get ("defaults", {}):
-                simple_rule = s_rule [:s] or "/"                
-                if simple_rule not in self.route_map:                                  
+                # implicit mount if not exist explicit one
+                simple_rule = s_rule [:s]
+                if not simple_rule:
+                    simple_rule = mount_prefix and mount_prefix + "/" or "/"
+
+                if simple_rule not in self.route_map:
                     options_ = copy.copy (options)
                     options_ ["argspec"] = proto [2:5]
                     self.add_route (simple_rule [len (mount_prefix or ''):], func, **options_)
             self._route_priority.append ((prefix, re_rule))
             self._route_priority.sort (key = lambda x: len (x [0]), reverse = True)
         
-        
         if "__proto__" in resource:
             methods = set (resource ["__proto__"][-1].get ("methods", []))
         else:
@@ -302,14 +306,19 @@ class Router:
         for i in range (len(arglist)):
             an, at = a [i]
             if at == "int":
-                if arglist [i] != "me":
+                if arglist [i] in self.SPECIALS:
+                    kargs [an] = arglist [i]                  
+                else:
                     kargs [an] = int (arglist [i])
             elif at == "float":
-                if arglist [i] != "me":
+                if arglist [i] in self.SPECIALS:
+                    kargs [an] = arglist [i]
+                else:
                     kargs [an] = float (arglist [i])
             elif at == "path":
                 kargs [an] = unquote_plus (arglist [i])
-            else:        
+            else:
+                # why?? I don't know, either, use hypen
                 kargs [an] = unquote_plus (arglist [i]).replace ("_", " ")
                 
         return f, options, kargs
@@ -329,7 +338,10 @@ class Router:
                     if not path_info.startswith (prefix):
                         continue                    
                     protos = self.route_map_fancy [prefix][rule]
-                    method, options, kargs = self.verify_rule (path_info, rule, protos, command)
+                    try:
+                        method, options, kargs = self.verify_rule (path_info, rule, protos, command)
+                    except ValueError:
+                        return self, None, None, None, 400
                     if method:
                         break
         except AssertionError:
