@@ -105,19 +105,20 @@ class Executor:
 
 		for r in required:
 			if r not in karg:
-				raise HTTPError ("400 Bad Request", "parameter `{}` is missing")
+				raise HTTPError ("400 Bad Request", "parameter `{}` is missing".format (r))
 		if self.was.request.routable.get ("keywords"):
 			return
 		
 		for r in karg:
 			if r not in args:
-				raise HTTPError ("400 Bad Request", "parameter `{}` needn't")	
+				raise HTTPError ("400 Bad Request", "parameter `{}` needn't".format (r))	
 
 	def generate_content (self, method, _args, karg):
 		karg = self.parse_kargs (karg)		
 		try:
 			response = self.chained_exec (method, _args, karg)			
 		except TypeError:			
+			self.was.traceback ()
 			self.verify_args_lazy (karg) #lazy validating request parameters for respond 400
 			raise
 		return response
@@ -228,6 +229,7 @@ class Executor:
 				request.response.error (respcode, respcodes.get (respcode, "Undefined Error"))
 
 		if thing:
+			request.salt = current_app.salt			
 			request.routed = current_app.get_routed (thing)
 			request.routable = options
 		
